﻿using RimWorld;
using RimWorld.Planet;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;
using Verse;
using VFECore.Abilities;
using Ability = VFECore.Abilities.Ability;

namespace Outland
{
    public class Ability_ConvertCorpse : Ability
    {
        public Corpse targetCorpse;

        public override bool ValidateTarget(LocalTargetInfo target, bool showMessages = true)
        {
            targetCorpse = (Corpse)target.Cell.GetThingList(pawn.Map).First(t => t as Corpse != null) ?? null;
            return targetCorpse != null && targetCorpse.InnerPawn.Dead && targetCorpse.InnerPawn.RaceProps.IsFlesh;
        }

        public override void Cast(GlobalTargetInfo[] targets)
        {
            SpawnPawn(targetCorpse.Position, targetCorpse.Map);
            targetCorpse.Strip();
            MakeBlood(targetCorpse.Position, targetCorpse.Map);
            targetCorpse.Destroy();
            base.Cast(targets);
        }

        public void MakeBlood(IntVec3 pos, Map map)
        {
            if(targetCorpse.InnerPawn.RaceProps.BloodDef == null)
            {
                return;
            }

            List<IntVec3> list = GenAdjFast.AdjacentCells8Way(pos);

            for (int i = 0; i < list.Count(); i++)
            {
                if (pos.IsValid)
                {
                    FilthMaker.TryMakeFilth(list[i], targetCorpse.Map, targetCorpse.InnerPawn.RaceProps.BloodDef);
                }
            }
        }

        public void SpawnPawn(IntVec3 pos, Map map)
        {
            AbilityExtension_PawnSpawnSingle extS = (AbilityExtension_PawnSpawnSingle)AbilityModExtensions.Find(ame => ame is AbilityExtension_PawnSpawnSingle);
            if(extS != null && extS.pawnKind != null)
            {
                PawnKindDef kindDef = extS.pawnKind;
                SpawnPawn(kindDef, pos, map);
            }
            AbilityExtension_PawnSpawnMulti extM = (AbilityExtension_PawnSpawnMulti)AbilityModExtensions.Find(ame => ame is AbilityExtension_PawnSpawnMulti);
            if(extM != null && !extM.pawnKinds.NullOrEmpty())
            {
                if (extM.singleRandomDef)
                {
                    PawnKindDef kindDef = extM.pawnKinds.RandomElement();
                    int count = extM.count.RandomInRange;
                    for (int i = 0; i < count; i++)
                    {
                        SpawnPawn(kindDef, pos, map);
                    }
                }
                else
                {
                    int count = extM.count.RandomInRange;
                    for (int i = 0; i < count; i++)
                    {
                        PawnKindDef kindDef = extM.pawnKinds.RandomElement();
                        SpawnPawn(kindDef, pos, map);
                    }
                }
            }
        }

        public void SpawnPawn(PawnKindDef kindDef, IntVec3 pos, Map map)
        {
            PawnGenerationRequest request = new PawnGenerationRequest(kind: kindDef, faction: pawn.Faction, fixedBiologicalAge: 0f, forceGenerateNewPawn: true, canGeneratePawnRelations: false);
            Pawn newPawn = PawnGenerator.GeneratePawn(request);

            GenSpawn.Spawn(newPawn, pos, map);
        }
    }
}
