﻿using RimWorld;
using RimWorld.Planet;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;
using Verse;
using VFECore.Abilities;
using Ability = VFECore.Abilities.Ability;
using AbilityDef = VFECore.Abilities.AbilityDef;

namespace Outland
{
    public class AbilityExtension_Arcane : AbilityExtension_AbilityMod
    {
        public float GetEnergyUsedByPawn(Pawn pawn) => this.energyUsed * pawn.GetStatValue(OutlandDefOf.Outland_MagicAbilityCostMultiplier);

        public int experience = 0;

        public float energyUsed = 0f;

        public ClassDef arcaneClass;

        public AbilityDef baseAbility = null;

        public AbilityDef higherAbility = null;

        public int upgradeCost = 1;

        public int tier = 0;

        public bool IsBottomLevel => baseAbility == null;

        public bool IsMultiLevel => baseAbility != null || higherAbility != null;

        public override bool IsEnabledForPawn(Ability ability, out string reason)
        {
            reason = "Outland.AbilityDisableReasonArcaneEnergyLack".Translate();

            return ability.Hediff != null && ((Hediff_ArcaneAbilities)ability.Hediff).SufficientEnergyPresent(GetEnergyUsedByPawn(ability.pawn));
        }

        public override void Cast(GlobalTargetInfo[] targets, Ability ability)
        {
            base.Cast(targets, ability);

            Hediff_ArcaneAbilities arcaneClass = (Hediff_ArcaneAbilities)ability.pawn.health.hediffSet.GetFirstHediffOfDef(OutlandDefOf.Outland_ArcaneClass);
            arcaneClass.UseEnergy(GetEnergyUsedByPawn(ability.pawn));
            arcaneClass.ExpGain(experience);
        }

        public override string GetDescription(Ability ability) =>
            $"{"Outland.AbilityStatsArcaneEnergy".Translate()}: {GetEnergyUsedByPawn(ability.pawn)}".Colorize(Color.cyan);
    }
}
