﻿//using AlienRace;
using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;
using Verse;

namespace Outland
{
	public class CompUseEffect_AddArcaneClass : CompUseEffect_InstallImplant
	{
		public new CompProperties_UseEffectAddArcaneClass Props => (CompProperties_UseEffectAddArcaneClass)props;

		public override void DoEffect(Pawn user)
		{
			BodyPartRecord bodyPartRecord = user.RaceProps.body.GetPartsWithDef(Props.bodyPart).FirstOrFallback();
			if (bodyPartRecord != null)
			{
				Hediff firstHediffOfDef = user.health.hediffSet.GetFirstHediffOfDef(Props.hediffDef);
				if (firstHediffOfDef == null)
				{
					Hediff_ArcaneClass arcaneHediff = (Hediff_ArcaneClass)HediffMaker.MakeHediff(Props.hediffDef, user, bodyPartRecord);
					arcaneHediff.arcaneClass = Props.arcaneClass;

					user.health.AddHediff(arcaneHediff);
				}
				else if (Props.canUpgrade)
				{
					((Hediff_Level)firstHediffOfDef).ChangeLevel(1);
				}
			}
		}

		public override bool CanBeUsedBy(Pawn p, out string failReason)
		{
			//ThingDef_AlienRace race = p.def as ThingDef_AlienRace;
			if ((!p.IsFreeColonist || p.HasExtraHomeFaction()) && !Props.allowNonColonists)
			{
				failReason = "Outland.IncapableOfArcaneReason_NotAllowedForNonColonists".Translate();
				return false;
			}
			//if (!race.alienRace.compatibility.IsSentient)
			//{
			//	failReason = "Outland.IncapableOfArcaneReason_RaceSentience".Translate() + ": " + p.def.LabelCap;
			//	return false;
			//}
			Hediff existingImplant = GetExistingImplant(p);
			if (existingImplant != null)
			{
				if (!Props.canUpgrade)
				{
					failReason = "Outland.IncapableOfArcaneReason_AlreadyKnown".Translate();
					return false;
				}
				Hediff_Level hediff_Level = (Hediff_Level)existingImplant;
				if (hediff_Level.level >= hediff_Level.def.maxSeverity)
				{
					failReason = "Outland.IncapableOfArcaneReason_AlreadyMaxLevel".Translate();
					return false;
				}
			}
			failReason = null;
			return true;
		}
	}
}
