﻿using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;
using Verse;

using VFECore.Abilities;

namespace Outland
{
	public class CompUseEffect_LearnSpell : CompUseEffect_GiveAbility
	{
		public new CompProperties_UseEffectLearnSpell Props => (CompProperties_UseEffectLearnSpell)props;

		public override bool CanBeUsedBy(Pawn p, out string failReason)
		{
			if (!base.CanBeUsedBy(p, out failReason))
			{
				return false;
			}
			Hediff_ArcaneClass arcaneHediff = p.health.hediffSet.GetFirstHediffOfDef(OutlandDefOf.Outland_ArcaneClass) as Hediff_ArcaneClass;
			if (arcaneHediff == null)
			{
				failReason = "Outland.IncapableOfArcaneReason_NotMage".Translate();
				return false;
			}
			if (arcaneHediff.arcaneClass != null && Props.requiredClass != arcaneHediff.arcaneClass)
			{
				failReason = "Outland.IncapableOfArcaneReason_WrongClass".Translate(arcaneHediff.arcaneClass.LabelCap);
				return false;
			}
			CompAbilities comp = p.GetComp<CompAbilities>();
			if (comp != null && comp.HasAbility(Props.ability) || !Props.incompatibleAbilities.Where(ia => comp.HasAbility(ia)).EnumerableNullOrEmpty())
            {
				failReason = "Outland.IncapableOfArcaneReason_AlreadyKnowSpell".Translate();
				return false;
            }
			failReason = null;
			return true;
		}
	}
}
