﻿using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;
using Verse;

using VFECore.Abilities;
using AbilityDef = VFECore.Abilities.AbilityDef;

namespace Outland
{
    public class Hediff_ArcaneAbilities : Hediff_AbilitiesWithLevels
	{
        public float arcaneEnergy;

		public override string Label => "Outland.ArcaneMage".Translate();

        public override void PostAdd(DamageInfo? dinfo)
		{
			base.PostAdd(dinfo);
			arcaneEnergy = pawn.GetStatValue(OutlandDefOf.Outland_ArcaneEnergyMax);
		}

		public void UseEnergy(float energyUsed)
		{
			arcaneEnergy -= energyUsed;
			arcaneEnergy = Mathf.Min(arcaneEnergy, pawn.GetStatValue(OutlandDefOf.Outland_ArcaneEnergyMax));
		}

		public bool SufficientEnergyPresent(float energyWanted)
		{
			return arcaneEnergy >= energyWanted;
		}

        public override void ExpGain(float xp)
        {
            base.ExpGain(xp *= OutlandMod.settings.arcane_expMulti);
        }

        public override void Tick()
		{
			base.Tick();
			arcaneEnergy += pawn.GetStatValue(OutlandDefOf.Outland_ArcaneEnergyRecoveryRate) / 60f;
			arcaneEnergy = Mathf.Min(arcaneEnergy, pawn.GetStatValue(OutlandDefOf.Outland_ArcaneEnergyMax));
		}

		public override IEnumerable<Gizmo> DrawGizmos()
		{
			yield return new Gizmo_ArcaneEnergyStatus
			{
				arcaneHediff = this
			};
		}

        public override void ExposeData()
		{
			base.ExposeData();
			Scribe_Values.Look(ref arcaneEnergy, "arcaneEnergy", 0f);
		}
	}
}
