﻿using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;
using Verse;

using VFECore.Abilities;
using AbilityDef = VFECore.Abilities.AbilityDef;

namespace Outland
{
    public class Hediff_AbilitiesWithLevels : Hediff_Abilities
    {
        public const float lvlConstant = 0.1f;
        public const float lvlPower = 2f;

        public float experienceInt = 0f;
        public float experienceLastTick = -1f;
        public int levelInt = 0;
        public int bonusSkillPoints = 0;
        public int spentSkillPoints = 0;
        public List<ClassDef> knownClasses = new List<ClassDef>();

        public override void ExposeData()
        {
            base.ExposeData();
            Scribe_Values.Look(ref experienceInt, "experienceInt", 0f);
            Scribe_Values.Look(ref experienceLastTick, "experienceLastTick", -1f);
            Scribe_Values.Look(ref levelInt, "levelInt", 0);
            Scribe_Values.Look(ref bonusSkillPoints, "bonusSkillPoints", 0);
            Scribe_Values.Look(ref spentSkillPoints, "spentSkillPoints", 0);
            Scribe_Collections.Look(ref knownClasses, "knownClasses");
        }

        public float PercentToNextLevel => Mathf.Clamp(((experienceInt - ExpForCurrentLevel) / TotalExpForNextLevel), 0f, 1f);

        public float ExpForNextLevel => TotalExpForNextLevel - ExpForCurrentLevel;

        public float TotalExpForNextLevel => ExpForGivenLevel(levelInt + 1);

        public float ExpForCurrentLevel => ExpForGivenLevel(levelInt);

        public float ExpForGivenLevel(int lvl)
        {
            if (lvl < 1)
            {
                return 0f;
            }
            //return OutlandMod.settings.arcane_expBase + (lvl * OutlandMod.settings.arcane_expMulti);
            // lvl / const) ^ pow
            return Mathf.Pow(lvl / lvlConstant, lvlPower);
        }

        public int LevelFromGivenExp(float exp)
        {
            if(exp < ExpForGivenLevel(1))
            {
                return 1;
            }
            //return Mathf.FloorToInt((exp - OutlandMod.settings.arcane_expBase) / OutlandMod.settings.arcane_expMulti);
            return Mathf.FloorToInt(lvlConstant * Mathf.Pow(exp, 1 / lvlPower));
        }

        public int AvailableSkillPoints => SkillPointsForLevel - spentSkillPoints;

        public int SkillPointsForLevel => (Mathf.RoundToInt(levelInt * OutlandMod.settings.arcane_pointsPerLevel) + bonusSkillPoints) - 1;

        public void SpendSkillPoints(int spent)
        {
            spentSkillPoints += spent;
        }

        public override void Tick()
        {
            base.Tick();

            if(experienceLastTick != experienceInt)
            {
                CalcLevel();
            }
            experienceLastTick = experienceInt;
        }

        public void CalcLevel()
        {
            levelInt = LevelFromGivenExp(experienceInt);
            if(level != levelInt)
            {
                SetLevelTo(levelInt);
            }
        }

        public virtual void ExpGain(float xp)
        {
            experienceInt += xp;
        }

        public override void GiveRandomAbilityAtLevel(int? forLevel = null)
        {
            if(forLevel > 1)
            {
                return;
            }
            GiveRandomAbilityAtLevelForClass(forLevel);
        }

        public void GiveRandomAbilityAtLevelForClass(int? forLevel = null)
        {
            if (giveRandomAbilities)
            {
                forLevel = new int?(forLevel ?? level);
                CompAbilities comp = pawn.GetComp<CompAbilities>();
                for (int i = 0; i < knownClasses.Count; i++)
                {
                    ClassDef arcaneClass = knownClasses[i];
                    if (!arcaneClass.defaultSpells.NullOrEmpty())
                    {
                        if (arcaneClass.learnAllDefaults)
                        {
                            for (int j = 0; j < arcaneClass.defaultSpells.Count; j++)
                            {
                                comp.GiveAbility(arcaneClass.defaultSpells[j]);
                            }
                        }
                        else
                        {
                            comp.GiveAbility(arcaneClass.defaultSpells.RandomElement());
                        }
                        return;
                    }
                }
            }
        }
    }
}
