﻿using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;
using Verse;

namespace Outland
{
    public abstract class ITab_Ability : ITab
	{
		public Vector2 scrollPosition = Vector2.zero;

		public float scrollViewHeight;

		public const float TopPadding = 20f;

		public const float ThingIconSize = 28f;

		public const float ThingRowHeight = 28f;

		public const float ThingLeftX = 36f;

		public const float StandardLineHeight = 22f;

		public const float InitialHeight = 450f;

		public virtual Hediff_AbilitiesWithLevels ClassHediff => null;

        public Pawn SelPawnForHediff
		{
			get
			{
				if (base.SelPawn != null)
				{
					return base.SelPawn;
				}
				if (base.SelThing is Corpse corpse)
				{
					return corpse.InnerPawn;
				}
				throw new InvalidOperationException("Class tab on non-pawn non-corpse " + base.SelThing);
			}
		}

		public bool CanControl
		{
			get
			{
				Pawn selPawnForHediff = SelPawnForHediff;
				if (selPawnForHediff.Downed || selPawnForHediff.InMentalState)
				{
					return false;
				}
				if (selPawnForHediff.Faction != Faction.OfPlayer && !selPawnForHediff.IsPrisonerOfColony)
				{
					return false;
				}
				if (selPawnForHediff.IsPrisonerOfColony && selPawnForHediff.Spawned && !selPawnForHediff.Map.mapPawns.AnyFreeColonistSpawned)
				{
					return false;
				}
				if (selPawnForHediff.IsPrisonerOfColony && (PrisonBreakUtility.IsPrisonBreaking(selPawnForHediff) || (selPawnForHediff.CurJob != null && selPawnForHediff.CurJob.exitMapOnArrival)))
				{
					return false;
				}
				return true;
			}
		}

		public override bool IsVisible => CanControl && ClassHediff != null;

		public ITab_Ability()
		{
			size = new Vector2(540f, 510f);
		}
	}
}
