﻿using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;
using Verse;

namespace Outland
{
    public class Comp_TomeStorage : ThingComp, IThingHolder
    {
        public CompProperties_TomeStorage Props => (CompProperties_TomeStorage)props;

        public ThingOwner innerContainer;

        public List<TomeBase> leftToLoad = new List<TomeBase>();

        public bool autoLoad = true;

        public List<TomeBase> tmpTomes = new List<TomeBase>();
        public List<AbilityTome> tmpAbilityTomes = new List<AbilityTome>();
        public List<ClassTome> tmpClassTomes = new List<ClassTome>();

        public static readonly CachedTexture ejectTex = new CachedTexture("UI/Gizmos/EjectAll");

        public bool StorageTabVisible => true;

        public bool Full => ContainedTomes.Count >= Props.maxCapacity;

        public bool CanLoadMore
        {
            get
            {
                if (!Full)
                {
                    return ContainedTomes.Count + leftToLoad.Count < Props.maxCapacity;
                }
                return false;
            }
        }

        public List<TomeBase> ContainedTomes
        {
            get
            {
                tmpTomes.Clear();
                for(int i = 0; i < innerContainer.Count; i++)
                {
                    if(innerContainer[i] is TomeBase)
                    {
                        tmpTomes.Add((TomeBase)innerContainer[i]);
                    }
                }
                return tmpTomes;
            }
        }

        public List<AbilityTome> ContainedAbilityTomes
        {
            get
            {
                tmpAbilityTomes.Clear();
                for (int i = 0; i < innerContainer.Count; i++)
                {
                    if (innerContainer[i] is AbilityTome || innerContainer[i] is ClassTome)
                    {
                        tmpAbilityTomes.Add((AbilityTome)innerContainer[i]);
                    }
                }
                return tmpAbilityTomes;
            }
        }

        public List<ClassTome> ContainedClassTomes
        {
            get
            {
                tmpClassTomes.Clear();
                for (int i = 0; i < innerContainer.Count; i++)
                {
                    if (innerContainer[i] is ClassTome)
                    {
                        tmpClassTomes.Add((ClassTome)innerContainer[i]);
                    }
                }
                return tmpClassTomes;
            }
        }

        public override void PostPostMake()
        {
            base.PostPostMake();
            innerContainer = new ThingOwner<TomeBase>(this);
        }

        public void GetChildHolders(List<IThingHolder> outChildren)
        {
            ThingOwnerUtility.AppendThingHoldersFromThings(outChildren, GetDirectlyHeldThings());
        }

        public ThingOwner GetDirectlyHeldThings()
        {
            return innerContainer;
        }

        public override void PostDestroy(DestroyMode mode, Map previousMap)
        {
            innerContainer.ClearAndDestroyContents();
            base.PostDestroy(mode, previousMap);
        }

        public override void PostDrawExtraSelectionOverlays()
        {
            for(int i = 0; i < leftToLoad.Count; i++)
            {
                if(leftToLoad[i].Map == parent.Map)
                {
                    GenDraw.DrawLineBetween(parent.DrawPos, leftToLoad[i].DrawPos);
                }
            }
        }

        public override void CompTick()
        {
            innerContainer.ThingOwnerTick();
        }

        public override void CompTickLong()
        {
            innerContainer.ThingOwnerTickLong();
        }

        public override void CompTickRare()
        {
            innerContainer.ThingOwnerTickRare();
        }

        public void EjectContents(Map destMap = null)
        {
            if(destMap == null)
            {
                destMap = parent.Map;
            }
            IntVec3 dropLoc = (parent.def.hasInteractionCell ? parent.InteractionCell : parent.Position);
            innerContainer.TryDropAll(dropLoc, destMap, ThingPlaceMode.Near);
        }

        public override IEnumerable<Gizmo> CompGetGizmosExtra()
        {
            foreach(Gizmo gizmo in base.CompGetGizmosExtra())
            {
                yield return gizmo;
            }
            if (DebugSettings.ShowDevGizmos)
            {
                IEnumerable<TomeDef> allTomes = DefDatabase<TomeDef>.AllDefs;
                if (allTomes.EnumerableNullOrEmpty())
                {
                    yield break;
                }
                List<TomeDef> allTomesList = allTomes.ToList();
                yield return new Command_Action
                {
                    defaultLabel = "DEV: Fill with new Tomes",
                    action = delegate
                    {
                        innerContainer.ClearAndDestroyContents();
                        for (int i = 0; i < Props.maxCapacity; i++)
                        {
                            innerContainer.TryAdd(ThingMaker.MakeThing(allTomesList.RandomElement()));
                        }
                    }
                };
            }
        }

        public override string CompInspectStringExtra()
        {
            return "Outland.TomesStored".Translate() + $": {innerContainer.Count} / {Props.maxCapacity}\n" + "CasketContains".Translate() + ": " + innerContainer.ContentsString.CapitalizeFirst();
        }

        public override void PostExposeData()
        {
            base.PostExposeData();
            Scribe_Deep.Look(ref innerContainer, "innerContainer", this);
            Scribe_Collections.Look(ref leftToLoad, "leftToLoad", LookMode.Reference);
            Scribe_Values.Look(ref autoLoad, "autoLoad", defaultValue: true);
        }
    }
}
