﻿using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;
using Verse;

namespace Outland
{
    public class Hediff_SoulTrap : HediffWithComps
    {
        public bool soulReaped = false;

        public override void ExposeData()
        {
            base.ExposeData();
            Scribe_Values.Look(ref soulReaped, "soulReaped");
        }

        public override bool ShouldRemove
        {
            get
            {
                if (soulReaped)
                {
                    return false;
                }
                return base.ShouldRemove;
            }
        }

        public override void Notify_PawnKilled()
        {
            base.Notify_PawnKilled();

            if (soulReaped)
            {
                return;
            }

            Thing soulGem = null;
            if (pawn.RaceProps.Humanlike)
            {
                soulGem = ThingMaker.MakeThing(OutlandDefOf.Outland_SoulGemBlack);
            }
            else
            {
                if (pawn.RaceProps.trainability == TrainabilityDefOf.None)
                {
                    if (pawn.BodySize < 0.8f)
                    {
                        soulGem = ThingMaker.MakeThing(OutlandDefOf.Outland_SoulGemPetty);
                    }
                    else
                    {
                        soulGem = ThingMaker.MakeThing(OutlandDefOf.Outland_SoulGemLesser);
                    }
                }
                else if (pawn.RaceProps.trainability == TrainabilityDefOf.Intermediate)
                {
                    if (pawn.BodySize < 1.0f)
                    {
                        soulGem = ThingMaker.MakeThing(OutlandDefOf.Outland_SoulGemCommon);
                    }
                    else
                    {
                        soulGem = ThingMaker.MakeThing(OutlandDefOf.Outland_SoulGemGreater);
                    }
                }
                else if (pawn.RaceProps.trainability == TrainabilityDefOf.Advanced)
                {
                    if (pawn.BodySize < 2.0f)
                    {
                        soulGem = ThingMaker.MakeThing(OutlandDefOf.Outland_SoulGemGreater);
                    }
                    else
                    {
                        soulGem = ThingMaker.MakeThing(OutlandDefOf.Outland_SoulGemGrand);
                    }
                }
                else
                {
                    if(pawn.BodySize < 0.65f)
                    {
                        soulGem = ThingMaker.MakeThing(OutlandDefOf.Outland_SoulGemPetty);
                    }
                    else if(pawn.BodySize < 1.0f)
                    {
                        soulGem = ThingMaker.MakeThing(OutlandDefOf.Outland_SoulGemLesser);
                    }
                    else if(pawn.BodySize < 1.5f)
                    {
                        soulGem = ThingMaker.MakeThing(OutlandDefOf.Outland_SoulGemCommon);
                    }
                    else if(pawn.BodySize < 2.0f)
                    {
                        soulGem = ThingMaker.MakeThing(OutlandDefOf.Outland_SoulGemGreater);
                    }
                    else
                    {
                        soulGem = ThingMaker.MakeThing(OutlandDefOf.Outland_SoulGemGrand);
                    }
                }
            }

            if(soulGem != null)
            {
                GenPlace.TryPlaceThing(soulGem, pawn.Position, pawn.Map, ThingPlaceMode.Direct);
                soulReaped = true;
            }
        }
    }
}
